local addonName, ns = ...
local _G = _G
local date = _G.date
local type = _G.type
local MAJOR_VERSION = addonName
local MINOR_VERSION = type('20110818212042') == 'number' and '20110818212042' or date('%Y%m%d%H%M%S')

local LibStub = _G.LibStub
local AceAddon = LibStub('AceAddon-3.0')
local addon = AceAddon:NewAddon(MAJOR_VERSION, 'AceConsole-3.0' ,'AceEvent-3.0' ,'AceHook-3.0')
ns.addon = addon
--local L = LibStub('AceLocale-3.0'):GetLocale(MAJOR_VERSION)

local UnitXPMax = _G.UnitXPMax
local UnitLevel = _G.UnitLevel
local UnitGetGuildXP = _G.UnitGetGuildXP
local GetGuildLevel = _G.GetGuildLevel

local XP_PER_LEVEL = ns.XP_PER_LEVEL

--- Setup debug stuff.
do
    local libDebugLog = LibStub:GetLibrary('LibDebugLog-1.0',true)
    if libDebugLog then
        libDebugLog:Embed(addon)
    else
        function addon:Debug (...)
        end
    end
end

local chatShort = "glb"
local chatLong = MAJOR_VERSION
local addonDbName = MAJOR_VERSION .. 'DB'

addon.optionsTable = {
    name = MAJOR_VERSION,
    handler = addon,
    type = 'group',
    args = {
    },
}

addon.defaults = {
    char = {
    },
}

--- Called for chat commands.
-- @param input the chat command
function addon:ChatCommand(input)
    if not input or input:trim() == '' then
        InterfaceOptionsFrame_OpenToCategory(self.optionsFrame)
    else
        LibStub('AceConfigCmd-3.0').HandleCommand(MAJOR_VERSION, chatShort, MAJOR_VERSION, input)
    end
end

--- Called when the addon is loaded
-- (Ace3 does this)
function addon:OnInitialize()
    self.db = LibStub('AceDB-3.0'):New(addonDbName, addon.defaults)
    LibStub('AceConfig-3.0'):RegisterOptionsTable(MAJOR_VERSION, self.optionsTable)
    self.optionsFrame = LibStub('AceConfigDialog-3.0'):AddToBlizOptions(MAJOR_VERSION, MAJOR_VERSION)
    self:RegisterChatCommand(chatShort, 'ChatCommand')
    self:RegisterChatCommand(chatLong, 'ChatCommand')

    self:sanityCheck()
end

function addon:sanityCheck()
    -- sanitiy check
    local unitXPMax = UnitXPMax('player')
    if unitXPMax ~= 0 then
        if unitXPMax ~= XP_PER_LEVEL[UnitLevel('player')] then
            error("The Player XP database is incorrect, please report this. Expected XP for level "
               .. UnitLevel('player') .. " is " .. XP_PER_LEVEL[UnitLevel('player')] .. " but UnitXPMax returned "
               .. unitXPMax)
        else
            self:Debug("Player xp sanity check passed")
        end
    end

    local guildXP, guildXPRem = UnitGetGuildXP('player')
    local guildXPMax = guildXP + guildXPRem
    local guildLevel = GetGuildLevel('player')
    if guildXPMax then
        if XP_PER_GUILD_LEVEL[guildLevel]
                      and guildXPMax ~= 0
                      and guildXPMax ~= XP_PER_GUILD_LEVEL[guildLevel] then
            error("The Guild XP database is incorrect, please report this. Expected XP for level "
               .. guildLevel .. " is " .. XP_PER_GUILD_LEVEL[guildLevel] .. " but game returned "
               .. guildXPMax)
        else
            self:Debug("Guild XP sanity check passed")
        end
    end
end


--- Called when the addon is enabled.
function addon:OnEnable()
    self:RegisterEvent('PLAYER_XP_UPDATE', 'sanityCheck')
    self:RegisterEvent('GUILD_XP_UPDATE',  'sanityCheck')
end

--- Called when the addon is disabled
function addon:OnDisable()
end

function addon:merge(t1, t2)
    local result = {}
    for k,v in pairs(t1) do
        if not t2[k] then
            --ChatFrame1:AddMessage("Copy key " .. k .. " right value " .. tostring(v))
            result[k] = self:deepCopy(v)
        end
    end
    for k,v in pairs(t2) do
        if t1[k] then
            if type(t1[k]) == 'table' then
                assert(type(v) == 'table')
                --ChatFrame1:AddMessage("Merging table key " .. k .. " values " .. tostring(t1[k]) .. " and " .. tostring(v))
                result[k] = self:merge(t1[k],v)
                --ChatFrame1:AddMessage("Merged  table key " .. k .. " values " .. tostring(t1[k]) .. " and " .. tostring(v))
            elseif type(t1[k]) == 'function' then
                assert(type(v) == 'function')
                --ChatFrame1:AddMessage("Merging function key " .. k .. " values " .. tostring(t1[k]) .. " and " .. tostring(v))
                result[k] = v
                assert(not result['super'..k])
                result['super'..k] = t1[k]
            else
                --ChatFrame1:AddMessage("Overwriting key " .. k .. " value " .. tostring(t1[k]) .. " with " .. tostring(v))
                result[k] = v
            end
        else
            --ChatFrame1:AddMessage("Copy key " .. k .. " left value " .. tostring(v))
            result[k] = self:deepCopy(v)
        end
    end
    return result
end

function addon:deepCopy(arg)
    if type(arg) == 'table' then
        local result = {}
        for k,v in pairs(arg) do
            result[k] = self:deepCopy(v)
        end
        return result
    else
        return arg
    end
end

function addon:formatNum(num)
    local first = string.sub(num,1,1)
    local middle = string.sub(num,2)
    return first .. string.reverse(middle):gsub('(%d%d%d)','%1.'):reverse()
end