local _G = _G
local date = _G.date
local MAJOR_VERSION = 'GraveLevelBar'
local MINOR_VERSION = type('20110813192702') == 'number' and '20110813192702' or date('%Y%m%d%H%M%S')
local MODULE_NAME = 'PlayerRep'

local LibStub = _G.LibStub
local AceAddon = LibStub('AceAddon-3.0')
local addon = AceAddon:GetAddon(MAJOR_VERSION)

local module = addon:createRepModule(MODULE_NAME, 'AceEvent-3.0')

local min = _G.min
local type = _G.type

-- start xp
--[[
function module:getStartXPMin()
    return 0
end

function module:getStartLevelMin()
    return 4
end
]]

module.optionsTable.args.factionId = {
    name = "Faction",
    type = 'input',
    validate = 'checkFaction',
    get = 'getFaction',
    set = 'setFaction',
}

function module:getFaction(info)
    return self:getOptions().factionId
end

function module:setFaction(info,id)
    self:getOptions().factionId = id
    self:Invalidate()
end

function module:checkFaction(info, input)
    if (not input) or (not type(input) == 'number') then
       return "Not a number"
    end
    -- do a protected call, if it suceeds this is a valid faction
    if pcall(function() GetFactionInfoByID(input) end) then
        local name = GetFactionInfoByID(input)
        addon:Debug(input, name, "is a faction")

        return true
    else
        addon:Debug(input,"is no faction")
        return "Not a valid faction"
    end
end

--function module:getFactions()
--
--end

module.oldEnable = module.OnEnable
function module:OnEnable()
    self:oldEnable()
    self:RegisterEvent('UPDATE_FACTION', 'Invalidate')
end

function module:Redraw()

    local options = self:getOptions()
    local barCache = self.cache

    --local factionID = options.factionID


    local currentGlobalXP = self:getCurrentXP()

    -- update max settings if necessary
    if options.targetXP and currentGlobalXP > options.targetXP then
       options.targetXP = currentGlobalXP
    end
    if options.startXP and currentGlobalXP < options.startXP then
        options.startXP = currentGlobalXP
    end

    local startGlobalXP  = options.startXP  or currentGlobalXP
    local targetGlobalXP = options.targetXP or currentGlobalXP

    local currentGlobalXPRatio =   (currentGlobalXP-startGlobalXP)/(targetGlobalXP-startGlobalXP)

    -- create textures
    assert(barCache and barCache.frame)
    local frame = barCache.frame
    local right = frame:GetRight()
    local left = frame:GetLeft()
    if not (right and left) then
        self:Invalidate()
        return
    end
    local width = frame:GetRight() - frame:GetLeft()
    -- texture for curent xp
    self:DrawPrimaryBar(currentGlobalXPRatio)

    -- textures and numbers for levels
    barCache.levelTextures = barCache.levelTextures or {}
    barCache.levelStrings = barCache.levelStrings or {}
    local levelTextures = barCache.levelTextures
    local levelStrings = barCache.levelStrings
    for level = 1,#REP_DATA do
        local xpForLeveL = self:calcXpForLevel(level)
        if startGlobalXP <= xpForLeveL and xpForLeveL <= targetGlobalXP then
            local levelGlobalXPRatio = (xpForLeveL-startGlobalXP)/(targetGlobalXP-startGlobalXP)
            -- textures
            levelTextures[level] = levelTextures[level] or frame:CreateTexture(frame:GetName()..'Level'..level,'OVERLAY')
            local levelTexture = levelTextures[level]
            levelTexture:ClearAllPoints()
            levelTexture:SetPoint('TOP')
            levelTexture:SetPoint('BOTTOM')
            if levelGlobalXPRatio < 1.0 then
                levelTexture:SetPoint('LEFT',frame,'LEFT',levelGlobalXPRatio * width,0)
            else
                levelTexture:SetPoint('RIGHT')
            end
            levelTexture:SetWidth(1)
            levelTexture:SetTexture(1,1,1,1)
            levelTexture:Show()
            -- numbers
            levelStrings[level] = levelStrings[level] or frame:CreateFontString(nil,'OVERLAY')
            local levelString = levelStrings[level]
            local isValid = levelString:SetFont('Fonts\\FRIZQT__.TTF',options.size)
            assert(isValid)
            levelString:ClearAllPoints()
            if levelGlobalXPRatio > 0 then
                levelString:SetPoint('RIGHT',levelTexture,'LEFT')
                levelString:SetJustifyH('RIGHT')
            else
                levelString:SetPoint('LEFT',levelTexture,'RIGHT')
                levelString:SetJustifyH('LEFT')
            end
            levelString:SetJustifyV('MIDDLE')
            levelString:SetText(REP_NAMES[level])
            levelString:Show()
        else
            if levelTextures[level] then
                levelTextures[level]:Hide()
            end
            if levelStrings[level] then
                levelStrings[level]:Hide()
            end
        end
    end

    --update options table
    self:updateOptionsTable()
end

function module:getCurrentXP()
    local factionId = self:getOptions().factionId
    if not factionId then return 0 end
    local isOk = pcall(function() GetFactionInfoByID(factionId) end)
    if not isOk then
        return 0
    end
    local name, description, standingID, barMin, barMax,
          barValue, atWarWith, canToggleAtWar, isHeader, isCollapsed,
          hasRep, isWatched, isChild = GetFactionInfoByID(factionId)
    assert(barMax - barMin == REP_DATA[standingID], "barMin=" .. barMin .. " barMax=" .. barMax)
    local levelRep = self:calcXpForLevel(standingID)
    assert(levelRep == barMin)
    local sum = barValue
    addon:Debug(name,standingID,levelRep, "value",barValue,"min", barMin,"max",barMax, sum)
    return sum
end

function module:fillTooltip(tooltip)
    local currentGlobalXP = self:getCurrentXP()
    local targetGlobalXP = self:getOptions().targetXP or self:getTargetXPMax()
    local startGlobalXP = self:getOptions().startXP
    local deltaXP = targetGlobalXP-startGlobalXP
    local percent = (currentGlobalXP-startGlobalXP)/deltaXP
    local strPercent = format('%.2f%%', percent*100)
    local strCurrent = addon:formatNum(math.floor(currentGlobalXP-startGlobalXP))
    local strDelta = addon:formatNum(math.ceil(deltaXP))
    tooltip:AddLine(strCurrent ..'/' .. strDelta .. '[' .. strPercent .. ']')
end

--module.defaults.targetXP = module:calcXpForLevel(module:getTargetLevelMax())